﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Windows.Forms;

namespace LaPucelleToolpack
{
	public class LZS
	{
		public LZS()
		{
			//decompress code from NISDecmp.exe
		}

		public byte[] decompressLZS(string sPath)
		{
			byte[] inFile = null;

			if (File.Exists(sPath))
			{
				using (FileStream fs = new FileStream(sPath, FileMode.Open))
				{
					if (fs.Length > 2147483647L)
					{
						throw new Exception("Filer larger than 2GB cannot be LZSS-compressed files.");
					}
				}

				inFile = File.ReadAllBytes(sPath);
				return decompressLZS(inFile);
			}
			return inFile;
		}

		public byte[] decompressLZS(byte[] inFile)
		{
			byte[] outFile = null;
			Tools tools = new Tools();

			using (MemoryStream msRead = new MemoryStream(inFile))
			{
				using (BinaryReader br = new BinaryReader(msRead))
				{
					string text = tools.ReadCString(br);

					if (text != "dat")
					{
						return null;
					}

					int USize = br.ReadInt32();
					int CSize = br.ReadInt32();

					outFile = new byte[USize];

					using (MemoryStream ms = new MemoryStream(outFile))
					{
						using (BinaryWriter bw = new BinaryWriter(ms))
						{
							if ((long)CSize != br.BaseStream.Length - 4L)
							{
								throw new Exception("value at 0x08 is not file length-0x04");
							}

							int num5 = 256;
							byte[] array = new byte[num5];
							for (int i = 0; i < num5; i++)
							{
								array[i] = 255;
							}
							int j = 0;
							int num6 = 0;
							byte b = br.ReadByte();
							if (br.ReadByte() != 0 || br.ReadByte() != 0 || br.ReadByte() != 0)
							{
								throw new Exception("padding bytes are not 0");
							}
							Console.Write("delimiter: 0x{0:x}. ", b);
							Console.WriteLine("Target output length: 0x{0:x}", USize);
							while (j < USize)
							{
								byte b2;
								try
								{
									b2 = br.ReadByte();
								}
								catch (EndOfStreamException)
								{
									Console.WriteLine("incomplete data or number at 0x04 isn't decompressed length");
									break;
								}
								if (b2 == b)
								{
									byte b3;
									try
									{
										b3 = br.ReadByte();
									}
									catch (EndOfStreamException)
									{
										Console.WriteLine("Incomplete data");
										break;
									}
									if (b3 == b) // not compressed, ...
									{
										bw.Write(b3);
										array[num6++] = b3;
										num6 %= num5;
										j++;
									}
									else
									{
										byte b4;
										try
										{
											b4 = br.ReadByte();
										}
										catch (EndOfStreamException)
										{
											Console.WriteLine("Incomplete data");
											break;
										}
										if (b3 >= b)
										{
											b3 -= 1;
										}
										for (int k = 0; k < (int)b4; k++)
										{
											b2 = array[num6 + num5 - (int)b3 & num5 - 1];
											bw.Write(b2);
											array[num6++] = b2;
											num6 %= num5;
											j++;
										}
									}
								}
								else
								{
									bw.Write(b2);
									array[num6++] = b2;
									num6 %= num5;
									j++;
								}
							}
							try
							{
								br.ReadByte();
								Console.WriteLine("Still some data left; value at 0x04 isn't decompressed length");
							}
							catch (EndOfStreamException)
							{
							}
							bw.Flush();
						}
					}
				}
			}
			return outFile;
		}

		public byte[] compressLZS(byte[] input)
		{
			byte[] data = new byte[input.Length];
			byte delim = 213;
			byte minCompress = 4;
			int maxBack = 254;
			int curPos = 0;
			byte[] curLookup;
			int iMatch;
			int iMatchLen;
			int curLookupPos;
			int CSize;
			byte b1;
			List<MatchTable> matches;
			MatchTable temp;

			using (MemoryStream msIn = new MemoryStream(input))
			{
			using (MemoryStream msOut = new MemoryStream(data))
			{
			using (BinaryReader br = new BinaryReader(msIn))
			{
			using (BinaryWriter bw = new BinaryWriter(msOut))
			{
				// write header
				bw.Write((byte)0x64); //d
				bw.Write((byte)0x61); //a
				bw.Write((byte)0x74); //t
				bw.Write((byte)0);
				bw.Write(input.Length);
				bw.Write((int)0); // compressed length, dummy
				bw.Write((int)delim);

				// start of file, alsways uncompressed 4 byte
				bw.Write(br.ReadByte());
				bw.Write(br.ReadByte());
				bw.Write(br.ReadByte());
				bw.Write(br.ReadByte());
				curPos = 4;

				bw.Write(br.ReadByte());
				curPos++;

				while (curPos < br.BaseStream.Length)
				{
					//get a lookup table
					if (curPos - maxBack < 0)
						curLookup = br.ReadBytes(curPos);
					else
					{
						if (br.BaseStream.Length - curPos >= maxBack)
							curLookup = br.ReadBytes(maxBack);
						else
							curLookup = br.ReadBytes((int)br.BaseStream.Length - curPos);
					}

					iMatch = -1;
					iMatchLen = 0;
					while (curLookup.Length >= minCompress && iMatch == -1)
					{
						curLookupPos = 0;
						matches = new List<MatchTable>();
						for (int j = 0; j < curLookup.Length; j++)
						{
							if (input[curPos - curLookup.Length + j] == curLookup[curLookupPos])
							{
								if (iMatch == -1) // found match
									iMatch = j;

								iMatchLen++;
								curLookupPos++;
							}
							else // nothing found
							{
								if (iMatchLen >= minCompress)
								{
									temp = new MatchTable();
									temp.count = iMatch;//curPos - (curPos - curLookup.Length + iMatch);
									temp.len = iMatchLen;
									matches.Add(temp);
									//break;
								}
								iMatch = -1;
								iMatchLen = 0;
								curLookupPos = 0;
							}
						}

						if (matches.Count > 0) // Find best compression len
						{
							FindBest(matches, ref iMatch, ref iMatchLen);
						}

						if (iMatchLen < minCompress) // nothing found
						{
							iMatch = -1;
							break;
						}
						else // found match, get count
						{
							iMatch = curPos - (curPos - curLookup.Length + iMatch);
							break;
						}	
					}

					if (iMatch >= 0 && iMatchLen > 0) // write compressed data
					{
						bw.Write(delim);

						if (iMatch < delim)
							bw.Write((byte)iMatch);
						else
							bw.Write((byte)(iMatch+1));

						bw.Write((byte)iMatchLen);
						curPos += iMatchLen;
						br.BaseStream.Position = curPos;
					}
					else // write uncompressed data
					{
						br.BaseStream.Position = curPos;

						b1 = br.ReadByte();
						bw.Write(b1);

						if (b1 == delim) 
							bw.Write(b1);

						curPos++;
					}
					Application.DoEvents();
				}

				//write compressed size into header
				CSize = (int)bw.BaseStream.Position;
				bw.BaseStream.Position = 0x08;
				bw.Write(CSize - 4);
			}
			}
			}
			}
			//resize compressed data, because buffer is to big
			Array.Resize(ref data, CSize);

			return data;
		}

		private void FindBest(List<MatchTable> matches, ref int count, ref int len)
		{
			len = matches.Max(r => r.len);

			for (int i = 0; i < matches.Count; i++)
			{
				if (matches[i].len == len)
					count = matches[i].count;
			}



		}

	}

	public class MatchTable
	{
		public MatchTable()
		{
			count = 0;
			len = 0;
		}

		public int count { get; set; }
		public int len { get; set; }
	}
}
