﻿using System;
using System.IO;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace LaPucelleToolpack
{
	public class NISPACK
	{
		public List<NISPACK_ENTRY> entries;
		private Tools tools;
		private Dictionary<string, uint> typeList;

		public NISPACK(Tools tool)
		{
			tools = tool;
			errorMsg = "";
			typeList = new Dictionary<string, uint>();
			typeList.Add("script01.dat", 0x3B43AE45);
			typeList.Add("script02.dat", 0x3B43AE45);
			typeList.Add("script03.dat", 0x3B43AE46);
			typeList.Add("script04.dat", 0x3B43AE46);
			typeList.Add("script05.dat", 0x3B43AE46);
			typeList.Add("script06.dat", 0x3B43AE47);
			typeList.Add("script07.dat", 0x3B43AE47);
			typeList.Add("script08.dat", 0x3B43AE47);
			typeList.Add("script09.dat", 0x3B43AE48);
			typeList.Add("script10.dat", 0x3B43AE48);
			typeList.Add("script11.dat", 0x3B43AE48);
			typeList.Add("script12.dat", 0x3B43AE49);
			typeList.Add("script15.dat", 0x3B43AE4A);
			typeList.Add("script16.dat", 0x3B43AE4A);
			typeList.Add("script17.dat", 0x3B43AE4B);
			typeList.Add("script18.dat", 0x3B43AE4B);
			typeList.Add("talk01_jp.dat", 0x3B43AE50);
			typeList.Add("talk02_jp.dat", 0x3B43AE50);
			typeList.Add("talk03_jp.dat", 0x3B43AE50);
			typeList.Add("talk04_jp.dat", 0x3B43AE50);
			typeList.Add("talk05_jp.dat", 0x3B43AE50);
			typeList.Add("talk06_jp.dat", 0x3B43AE50);
			typeList.Add("talk07_jp.dat", 0x3B43AE50);
			typeList.Add("talk08_jp.dat", 0x3B43AE50);
			typeList.Add("talk09_jp.dat", 0x3B43AE51);
			typeList.Add("talk10_jp.dat", 0x3B43AE51);
			typeList.Add("talk11_jp.dat", 0x3B43AE51);
			typeList.Add("talk12_jp.dat", 0x3B43AE51);
			typeList.Add("talk15_jp.dat", 0x3B43AE52);
			typeList.Add("talk16_jp.dat", 0x3B43AE52);
			typeList.Add("talk17_jp.dat", 0x3B43AE52);
			typeList.Add("talk18_jp.dat", 0x3B43AE52);
			typeList.Add("start_jp.lzs", 0x3C485544); //dlc0: 0x3C485544, dlc1: 0x3CAC5D41 
		}

		public int ExtractFiles(string sPath, string outDir)
		{
			if (File.Exists(sPath))
			{
				entries = new List<NISPACK_ENTRY>();
				NISPACK_ENTRY temp;

				using (BinaryReader br = new BinaryReader(File.OpenRead(sPath)))
				{
					// read header
					if (tools.ReadCString(br, 0xC) != "NISPACK")
					{
						errorMsg = "Error - file is not nispack!";
						return -1;
					}
					numFiles = br.ReadInt32();

					// read filetable
					for (int i = 0; i < numFiles; i++)
					{
						temp = new NISPACK_ENTRY();
						temp.name = tools.ReadCString(br, 0x20);
						temp.offset = br.ReadInt32();
						temp.size = br.ReadInt32();
						temp.unk1 = br.ReadInt32();

						entries.Add(temp);
					}

					//create dir
					if (Directory.Exists(outDir))
						Directory.Delete(outDir);

					Directory.CreateDirectory(outDir);

					// extract files
					for (int i = 0; i < numFiles; i++)
					{
						tools.DeleteFileIfExists(outDir + entries[i].name);
						br.BaseStream.Seek(entries[i].offset, SeekOrigin.Begin);
						File.WriteAllBytes(outDir + entries[i].name, br.ReadBytes(entries[i].size));
					}
				}
				return 0;
			}

			errorMsg = "Error - file does not exist!";
			return -1;
		}

		public void RepackFiles(string sPathFolder, string sPathFile)
		{
			if (Directory.Exists(sPathFolder))
			{
				Tools.SimpleFolder sf = new Tools.SimpleFolder(sPathFolder);
				tools.DeleteFileIfExists(sPathFile);

				List<int> offsets = new List<int>();
				int offset = 0;

				using (BinaryWriter bw = new BinaryWriter(File.OpenWrite(sPathFile)))
				{
					// write header
					bw.Write(Encoding.ASCII.GetBytes("NISPACK"));
					tools.WritePadding(bw, 5);
					bw.Write(sf.count);

					offset += tools.GetAlign(0x10 + (sf.count * 0x2C), 0x800);
					offsets.Add(offset);
					for (int i = 0; i < sf.count; i++)
					{
						bw.Write(Encoding.ASCII.GetBytes(sf.fName[i]));
						tools.WritePadding(bw, 0x20 - Encoding.ASCII.GetByteCount(sf.fName[i]));
						bw.Write(offset);
						bw.Write((int)sf.fSize[i]);
						bw.Write(typeList[sf.fName[i]]);
						offset += tools.GetAlign((int)sf.fSize[i], 0x800, true);
						offsets.Add(offset);
					}
					//tools.WritePadding(bw, bw.BaseStream.Position - offsets[0]);

					for (int i = 0; i < sf.count; i++)
					{
						bw.BaseStream.Seek(offsets[i], SeekOrigin.Begin);
						bw.Write(File.ReadAllBytes(sf.fPath[i]));
					}

				}

			}
		}

		public string errorMsg { get; set; }
		public int numFiles { get; set; }
	}

	public class NISPACK_ENTRY
	{
		public NISPACK_ENTRY()
		{
			name = "";
			offset = 0;
			size = 0;
			unk1 = 0;
		}

		public string name { get; set; }
		public int offset { get; set; }
		public int size { get; set; }
		public int unk1 { get; set; }
	}
}
