﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;

namespace LaPucelleToolpack
{
	public class START_DAT
	{
		public List<START_DAT_ENTRY> files;
		private Tools tools;

		public START_DAT(Tools tool)
		{
			tools = tool;
			errorMsg = "";
		}

		public int ExtractFiles(byte[] data, string outDir)
		{
			int start, tmp, offset;
			byte[] outFile = null;

			files = new List<START_DAT_ENTRY>();
			START_DAT_ENTRY temp;
			using (MemoryStream ms = new MemoryStream(data))
			{
			using (BinaryReader br = new BinaryReader(ms))
			{
				//read header
				numFiles = br.ReadInt32();
				br.BaseStream.Seek(0x10, SeekOrigin.Begin);
				start = (numFiles * 0x20) + 0x10;
				offset = start;

				//read filetable
				for (int i = 0; i < numFiles; i++)
				{
					temp = new START_DAT_ENTRY();
					tmp = br.ReadInt32();
					temp.name = tools.ReadCString(br, 0x1C);
					temp.size = tmp + start - offset;
					temp.offset = offset;
					files.Add(temp);
					offset = tmp + start;
				}
				
				//create dir
				if (Directory.Exists(outDir))
					Directory.Delete(outDir);

				Directory.CreateDirectory(outDir);

				//extract files
				for (int i = 0; i < numFiles; i++)
				{
					br.BaseStream.Seek(files[i].offset, SeekOrigin.Begin);
					outFile = br.ReadBytes(files[i].size);
					tools.DeleteFileIfExists(outDir + files[i].name);
					File.WriteAllBytes(outDir + files[i].name, outFile);
				}
				
			}
			}
			return 0;
			//errorMsg = "Error - file does not exist!";
			//return -1;
		}

		public byte[] RepackFiles(string sPathFolder)
		{
			byte[] data = null;
			if (Directory.Exists(sPathFolder))
			{
				Tools.SimpleFolder sf = new Tools.SimpleFolder(sPathFolder);
				List<int> offsets = new List<int>();
				int offset = 0;
				int start;
				this.numFiles = sf.count;

				using (MemoryStream ms = new MemoryStream())
				{
					using (BinaryWriter bw = new BinaryWriter(ms))
					{

						//write header
						bw.Write(this.numFiles);
						bw.BaseStream.Seek(0x10, SeekOrigin.Begin);
						start = (this.numFiles * 0x20) + 0x10;
						offset = start;

						//read filetable
						for (int i = 0; i < this.numFiles; i++)
						{
							offsets.Add(offset);
							offset += (int)sf.fSize[i];
							bw.Write(offset - start);
							bw.Write(Encoding.ASCII.GetBytes(sf.fName[i]));
							tools.WritePadding(bw, 0x1C - Encoding.ASCII.GetByteCount(sf.fName[i]));
						}

						//write files
						for (int i = 0; i < this.numFiles; i++)
						{
							bw.BaseStream.Seek(offsets[i], SeekOrigin.Begin);
							bw.Write(File.ReadAllBytes(sf.fPath[i]));
						}
					}
					data = ms.ToArray();
				}

			}
			return data;
		}

		public string errorMsg { get; set; }
		public int numFiles { get; set; }
	}
	public class START_DAT_ENTRY
	{
		public START_DAT_ENTRY()
		{
			offset = 0;
			size = 0;
			name = "";
		}
		public int offset { get; set; }
		public int size { get; set; }
		public string name { get; set; }
	}
}
