﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;

namespace LaPucelleToolpack
{
	public class SCRIPT_TALK
	{
		public List<SCRIPT_TALK_ENTRY> entries;
		Tools tools;

		public SCRIPT_TALK(Tools tool)
		{
			tools = tool;
			entries = new List<SCRIPT_TALK_ENTRY>();
			numEntries = 0;
			errorMsg = "";
		}

		public int ReadData(string sPath)
		{
			if (File.Exists(sPath))
			{
				SCRIPT_TALK_ENTRY temp;
				int start;

				using (BinaryReader br = new BinaryReader(File.OpenRead(sPath)))
				{
					// read header
					numEntries = br.ReadInt32();
					br.ReadInt32(); // dupe of numEntries
					start = (numEntries * 0x20) + 0x8;

					// read entries
					for (int i = 0; i < numEntries; i++)
					{
						temp = new SCRIPT_TALK_ENTRY();
						temp.ptrText = br.ReadInt32();

						for (int j = 0; j < 14; j++)
						{
							temp.flags[j] = br.ReadUInt16();
						}

						temp.ofsData = start + temp.ptrText;
						entries.Add(temp);
					}

					// calculate size
					for (int i = 0; i < numEntries -1; i++)
					{
						entries[i].szData = entries[i+1].ofsData - entries[i].ofsData;
					}
					entries[numEntries - 1].szData = (int)br.BaseStream.Length - entries[numEntries - 1].ofsData;

					// get script data
					for (int i = 0; i < numEntries; i++)
					{
						br.BaseStream.Seek(entries[i].ofsData, SeekOrigin.Begin);
						entries[i].data = br.ReadBytes(entries[i].szData);
					}

					return 0;
				}//end using BinaryReader
			}//end if
			errorMsg = "Error - file does not exist!";
			return -1;
		}

		public int WriteData(string sPath)
		{
			tools.DeleteFileIfExists(sPath);

			using (BinaryWriter bw = new BinaryWriter(File.OpenWrite(sPath)))
			{
				bw.Write(this.numEntries);
				bw.Write(this.numEntries);

				CalcOffsets();

				for (int i = 0; i < this.numEntries; i++)
				{
					bw.Write(this.entries[i].ptrText);

					for (int j = 0; j < 14; j++)
					{
						bw.Write(this.entries[i].flags[j]);
					}
				}

				for (int i = 0; i < numEntries; i++)
				{
					bw.BaseStream.Seek(entries[i].ofsData, SeekOrigin.Begin);
					bw.Write(entries[i].data);
				}

				return 0;
			}

			//errorMsg = "Error - file does not exist!";
			//return -1;
		}

		private void CalcOffsets()
		{
			int offset = 0;
			int start = (numEntries * 0x20) + 0x8;

			for (int i = 0; i < this.numEntries; i++)
			{
				this.entries[i].ptrText = offset;
				this.entries[i].ofsData = start + offset;
				offset += this.entries[i].data.Length;
			}
		}

		public string ReadTextScript(byte[] data)
		{
			string result = "";

			if (data.Length == 0)
				return result;

			List<byte> textData = new List<byte>();
			

			using (MemoryStream ms = new MemoryStream(data))
			{
				using (BinaryReader br = new BinaryReader(ms))
				{
					byte b1 = 0xFF;
					bool IsText = false;
					bool IsChoice = false;

					do
					{
						b1 = br.ReadByte();

						if (IsText == true)
						{
							if (b1 != 0x0)
							{
								textData.Add(b1);
								continue;
							}
							else
							{
								result += "\\Text;";
								result += Encoding.GetEncoding("shift-jis").GetString(textData.ToArray());
								result += "\r\n";
								IsText = false;

								if (br.BaseStream.Position < br.BaseStream.Length)
									continue;
							}
						}

						switch (b1)
						{
							case 0x00: break; // ignore 0x00
							case 0x01: IsText = true; textData = new List<byte>(); break; // start of text
							case 0x02: result += "\\n;"; break; // next dialog
							case 0x03: result += "\\end;"; break; // new dialog box
							case 0x05: result += "\\Face" + ReadScriptData(br, 2); break;
							case 0x06: result += "\\VoiceEnd" + ReadScriptData(br, 2); break; // Voice start in PS2
							case 0x07: result += "\\VoiceStart" + ReadScriptData(br, 2); break; // psp only
							case 0x09: result += "\\Label" + ReadScriptData(br, 2); break; // ???
							case 0x0A: result += "\\0A" + ReadScriptData(br, 2); break; // ???
							case 0x0B: result += "\\PPV" + ReadScriptData(br, 3); break; // ???
							case 0x0C: result += "\\Goto" + ReadScriptData(br, 2); break; // ???
							case 0x14:
								if (IsChoice == false)
								{
									result += "\\Choice" + ReadScriptData(br, 1); // ???
									IsChoice = true;
									break;
								}
								result += "\\Choice;"; break; // ???
							case 0xFF: result += "\\eof;"; break; // end of text
							case 0xC9: result += "\\C9" + ReadScriptData(br, 1); break; // ???
							default: result += string.Format("\\x;{0}", b1); break; // unknown
						}

						if (IsText == false && b1 != 0x00) // add new line to each command
							result += "\r\n";

					} while (br.BaseStream.Position < br.BaseStream.Length);
				}//end using BinaryReader
			}//end MemoryStream
			return result;
		}

		private string ReadScriptData(BinaryReader br, int len)
		{
			byte b1;
			int iTemp = 0;
			for (int i = 0; i < len; i++)
			{
				b1 = br.ReadByte();
				iTemp += (b1 << (i * 8));
			}
			return ";" + iTemp;
		}

		public byte[] WriteTextScript(string script, bool bWritePadding = true)
		{
			byte[] data = null;
			string[] lines = script.Split(new string[] { "\r\n" }, StringSplitOptions.None);
			string[] opcode;

			using (MemoryStream ms = new MemoryStream())
			{
				using (BinaryWriter bw = new BinaryWriter(ms))
				{
					foreach (string s in lines)
					{
						opcode = s.Split(';');

						switch(opcode[0])
						{
							case "\\Text": bw.Write((byte)0x01); 
								bw.Write(Encoding.GetEncoding("shift-jis").GetBytes(opcode[1]));
								if (bWritePadding == true)
									tools.WritePadding(bw, this.GetPadding(opcode[1]));
								bw.Write((byte)0x00);
								break;
							case "\\n": bw.Write((byte)0x02); break;
							case "\\end": bw.Write((byte)0x03); break;
							case "\\Face": bw.Write((byte)0x05); WriteScriptData(bw, opcode[1], 2); break;
							case "\\VoiceEnd": bw.Write((byte)0x06); WriteScriptData(bw, opcode[1], 2); break;
							case "\\VoiceStart": bw.Write((byte)0x07); WriteScriptData(bw, opcode[1], 2); break;
							case "\\Label": bw.Write((byte)0x09); WriteScriptData(bw, opcode[1], 2); break;
							case "\\0A": bw.Write((byte)0x0A); WriteScriptData(bw, opcode[1], 2); break;
							case "\\PPV": bw.Write((byte)0x0B); WriteScriptData(bw, opcode[1], 3); break;
							case "\\Goto": bw.Write((byte)0x0C); WriteScriptData(bw, opcode[1], 2); break;
							case "\\C9": bw.Write((byte)0xC9); WriteScriptData(bw, opcode[1], 1); break;
							case "\\eof": bw.Write((byte)0xFF); break;
							case "\\x": WriteScriptData(bw, opcode[1], 1); break;
							case "\\Choice":
								bw.Write((byte)0x14);
								if (opcode[1] != "")
									WriteScriptData(bw, opcode[1], 1);
								break;
							case "": break;
							default: throw new NotImplementedException();
						}
					}
					data = ms.ToArray();
				}
			}

			return data;
		}

		private void WriteScriptData(BinaryWriter bw, string value, int len)
		{
			ulong uvalue = Convert.ToUInt64(value);
            ulong temp1 = uvalue;

			for (int i = 0; i < len; i++)
			{
                uvalue = temp1;
				uvalue >>= (i * 8);
				bw.Write((byte)(uvalue & 0xFF));
			}
		}

		private int GetPadding(string text)
		{
			int len = 0;

			for (int i = 0; i < text.Length; i++)
			{
				if (text[i] == '\'') continue;
				if (text[i] == '(') continue;
				if (text[i] == ')') continue;
				if (text[i] == '!') continue;
				if (text[i] == '?') continue;
				if (text[i] == ' ') continue;

				if ((int)text[i] < 127) // only ASCII
				{
					len++;
				}
			}

			return len;
		}

		public int numEntries { get; set; }
		public string errorMsg { get; set; }
		
	}

	public class SCRIPT_TALK_ENTRY
	{
		public SCRIPT_TALK_ENTRY()
		{
			ptrText = 0;
			flags = new ushort[14];
		}

		// real data
		public int ptrText { get; set; }

		public ushort[] flags { get; set; }

		//extra data
		public int ofsData { get; set; }
		public int szData { get; set; }
		public byte[] data { get; set; }
	}
}
