﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.Drawing;

namespace LaPucelleToolpack
{
	public class Tools
	{

		[DllImport("msvcrt.dll", CallingConvention = CallingConvention.Cdecl)]
		static extern int memcmp(byte[] b1, byte[] b2, long count);

		public Tools()
		{

		}

		public bool OpenFileDlg(string sFilter, string sTitle, ref string sPath)
		{
			using (OpenFileDialog fOpenDlg = new OpenFileDialog())
			{
				fOpenDlg.Filter = sFilter;
				fOpenDlg.FilterIndex = 1;
				fOpenDlg.InitialDirectory = Application.StartupPath;
				fOpenDlg.Title = sTitle;

				if (fOpenDlg.ShowDialog() != DialogResult.OK)
				{
					return false;
				}

				sPath = fOpenDlg.FileName;

				if (string.IsNullOrEmpty(sPath))
					return false;
				else
					return true;
			}
		}

		public bool SaveFileDlg(string sFilter, string sTitle, string sFileName, ref string sPath)
		{
			using (SaveFileDialog fSaveDlg = new SaveFileDialog())
			{
				fSaveDlg.Filter = sFilter;
				fSaveDlg.FilterIndex = 1;
				fSaveDlg.InitialDirectory = Application.StartupPath;
				fSaveDlg.Title = sTitle;
				fSaveDlg.FileName = sFileName;

				if (fSaveDlg.ShowDialog() != DialogResult.OK)
					return false;

				sPath = fSaveDlg.FileName;

				if (string.IsNullOrEmpty(sPath))
					return false;
				else
					return true;
			}
		}

		public string ChooseFolder(string sDesc)
		{
			using (FolderBrowserDialog folders = new FolderBrowserDialog())
			{
				folders.SelectedPath = Directory.GetCurrentDirectory();
				folders.Description = sDesc;

				if (folders.ShowDialog() == DialogResult.OK)
				{
					return folders.SelectedPath;
				}
			}

			return "";
		}

		public string ReadCString(BinaryReader br, int MaxLength = -1, long lOffset = -1, Encoding enc = null)
		{
			string result = "";

			int Max;
			if (MaxLength == -1)
				Max = 4096;
			else
				Max = MaxLength;

			long fTemp = br.BaseStream.Position;
			byte bTemp = 0;
			int i = 0;

			if (lOffset > -1)
			{
				br.BaseStream.Seek(lOffset, SeekOrigin.Begin);
			}

			do
			{
				bTemp = br.ReadByte();
				if (bTemp == 0)
					break;
				i += 1;
			} while (i < Max);

			if (MaxLength == -1)
				Max = i+1;
			else
				Max = MaxLength;

			if (lOffset > -1)
			{
				br.BaseStream.Seek(lOffset, SeekOrigin.Begin);

				if (enc == null)
					result = Encoding.ASCII.GetString(br.ReadBytes(i));
				else
					result = enc.GetString(br.ReadBytes(i));

				br.BaseStream.Seek(fTemp, SeekOrigin.Begin);
			}
			else
			{
				br.BaseStream.Seek(fTemp, SeekOrigin.Begin);
				if (enc == null)
					result = Encoding.ASCII.GetString(br.ReadBytes(i));
				else
					result = enc.GetString(br.ReadBytes(i));

				br.BaseStream.Seek(fTemp + Max, SeekOrigin.Begin);
			}

			return result;
		}

		public string ReadUnicodeCString(BinaryReader br, int MaxLength = -1, long lOffset = -1, Encoding enc = null)
		{
			string result = "";

			int Max;
			if (MaxLength == -1)
				Max = 4096;
			else
				Max = MaxLength;

			long fTemp = br.BaseStream.Position;
			ushort usTemp = 0;
			int i = 0;

			if (lOffset > -1)
			{
				br.BaseStream.Seek(lOffset, SeekOrigin.Begin);
			}

			do
			{
				usTemp = br.ReadUInt16();
				if (usTemp == 0)
					break;
				i += 1;
			} while (i < Max);

			if (MaxLength == -1)
			{
				i *= 2; // unicode = 2 byte
				Max = i + 2;
			}
			else
				Max = MaxLength;

			if (lOffset > -1)
			{
				br.BaseStream.Seek(lOffset, SeekOrigin.Begin);

				if (enc == null)
					result = Encoding.Unicode.GetString(br.ReadBytes(i));
				else
					result = enc.GetString(br.ReadBytes(i));

				br.BaseStream.Seek(fTemp, SeekOrigin.Begin);
			}
			else
			{
				br.BaseStream.Seek(fTemp, SeekOrigin.Begin);
				if (enc == null)
					result = Encoding.Unicode.GetString(br.ReadBytes(i));
				else
					result = enc.GetString(br.ReadBytes(i));

				br.BaseStream.Seek(fTemp + Max, SeekOrigin.Begin);
			}

			return result;
		}

		public void DeleteFileIfExists(string sPath)
		{
			if (File.Exists(sPath))
				File.Delete(sPath);
		}

		public string GetPath(string input)
		{
			return Path.GetDirectoryName(input) + "\\" + Path.GetFileNameWithoutExtension(input);
		}

		public long GetEndian(byte[] src, int offset, int length)
		{
			long result = 0;

			if (length <= 0)
				return result;

			if (length > 8)
				return result;

			if (length == 1)
				return (long)src[offset++];

			int shift = (length-1) * 8;

			for (int i = 0; i < length; i++)
			{
				result += (long)(src[offset] << shift);
				shift -= 8;
				offset++;
			}
			return result;
		}

		public long GetEndian(byte[] src, ref int offset, int length)
		{
			long result = 0;

			if (length <= 0)
				return result;

			if (length > 8)
				return result;

			if (length == 1)
				return (long)src[offset++];

			int shift = (length - 1) * 8;

			for (int i = 0; i < length; i++)
			{
				result += (long)(src[offset] << shift);
				shift -= 8;
				offset++;
			}
			return result;
		}

		public bool ByteArrayCompare(byte[] b1, byte[] b2)
		{
			// Validate buffers are the same length.
			// This also ensures that the count does not exceed the length of either buffer.  
			return b1.Length == b2.Length && memcmp(b1, b2, b1.Length) == 0;
		}

		public static bool equals(sbyte[] a1, sbyte[] a2)
		{
			if (a1 == a2)
			{
				return true;
			}
			if ((a1 != null) && (a2 != null))
			{
				if (a1.Length != a2.Length)
				{
					return false;
				}
				for (int i = 0; i < a1.Length; i++)
				{
					if (a1[i] != a2[i])
					{
						return false;
					}
				}
				return true;
			}
			return false;
		}

		public Color ConvertRGB565(short data)
		{
			int r = ((data & 0xF800) >> 11) << 3;
			int g = ((data & 0x7E0) >> 5) << 2;
			int b = ((data & 0x1F)) << 3;

			return Color.FromArgb(r, g, b);
		}

		public Color ConvertRGB565(ushort data)
		{
			int r = ((data & 0xF800) >> 11) << 3;
			int g = ((data & 0x7E0) >> 5) << 2;
			int b = ((data & 0x1F)) << 3;

			return Color.FromArgb(r, g, b);
		}

		public Color ConvertRGB5(short data)
		{
			// format = Trrrrrgggggbbbbb
			int r = ((data & 0x7C00) >> 10) << 3;
			int g = ((data & 0x3E0) >> 5) << 3;
			int b = ((data & 0x1F)) << 3;

			return Color.FromArgb(r, g, b);
		}

		public Color ConvertRGB5(ushort data)
		{
			// format = Trrrrrgggggbbbbb
			int r = ((data & 0x7C00) >> 10) << 3;
			int g = ((data & 0x3E0) >> 5) << 3;
			int b = ((data & 0x1F)) << 3;

			return Color.FromArgb(r, g, b);
		}

		public Color ConvertRGB4A3(short data)
		{
			// format = xTTTrrrrggggbbbb
			int a = ((data & 0x7000) >> 12) << 4;
			int r = ((data & 0xF00) >> 8) << 4;
			int g = ((data & 0xF0) >> 4) << 4;
			int b = ((data & 0xF)) << 4;

			return Color.FromArgb(a, r, g, b);
		}

		public Color ConvertRGB4A3(ushort data)
		{
			// format = xTTTrrrrggggbbbb
			int a = ((data & 0x7000) >> 12) << 4;
			int r = ((data & 0xF00) >> 8) << 4;
			int g = ((data & 0xF0) >> 4) << 4;
			int b = ((data & 0xF)) << 4;

			return Color.FromArgb(a, r, g, b);
		}

		public Color ConvertRGB5A1(short data)
		{
			// format = trrrrrgggggbbbbb
			int r = ((data & 0x7C00) >> 10) << 3;
			int g = ((data & 0x3E0) >> 5) << 3;
			int b = ((data & 0x1F)) << 3;

			int a = (data & 0x8000) >> 15;

			if (a == 0)
				return Color.FromArgb(255, 0, 0, 0);
			else
				return Color.FromArgb(255, r, g, b);
		}

		public Color ConvertRGB5A1(ushort data)
		{
			// format = trrrrrgggggbbbbb
			int r = ((data & 0x7C00) >> 10) << 3;
			int g = ((data & 0x3E0) >> 5) << 3;
			int b = ((data & 0x1F)) << 3;

			int a = (data & 0x8000) >> 15;

			if (a == 0)
				return Color.FromArgb(255, 0, 0, 0);
			else
				return Color.FromArgb(255, r, g, b);
		}

		public int GetAlignCount(int input, int Align)
		{
			return Align - (input % Align);
		}

		public int GetAlign(int input, int Align, bool bAllowZero = false)
		{
			int result = input + GetAlignCount(input, Align);

			if (bAllowZero == true) // this will allow the 0 case, no padding is written
			{
				if (result == 0)
					result = input;
			}

			return result;
		}

		public void WritePadding(BinaryWriter bw, int count)
		{
			for (int i = 0; i < count; i++)
			{
				bw.Write((byte)0);
			}
		}

		public string HexDump(byte[] bytes, int bytesPerLine = 16)
		{
			if (bytes == null) return "<null>";
			int bytesLength = bytes.Length;

			char[] HexChars = "0123456789ABCDEF".ToCharArray();

			int firstHexColumn =
				  8                   // 8 characters for the address
				+ 3;                  // 3 spaces

			int firstCharColumn = firstHexColumn
				+ bytesPerLine * 3       // - 2 digit for the hexadecimal value and 1 space
				+ (bytesPerLine - 1) / 8 // - 1 extra space every 8 characters from the 9th
				+ 2;                  // 2 spaces 

			int lineLength = firstCharColumn
				+ bytesPerLine           // - characters to show the ascii value
				+ Environment.NewLine.Length; // Carriage return and line feed (should normally be 2)

			char[] line = (new String(' ', lineLength - 2) + Environment.NewLine).ToCharArray();
			int expectedLines = (bytesLength + bytesPerLine - 1) / bytesPerLine;
			StringBuilder result = new StringBuilder(expectedLines * lineLength);

			for (int i = 0; i < bytesLength; i += bytesPerLine)
			{
				line[0] = HexChars[(i >> 28) & 0xF];
				line[1] = HexChars[(i >> 24) & 0xF];
				line[2] = HexChars[(i >> 20) & 0xF];
				line[3] = HexChars[(i >> 16) & 0xF];
				line[4] = HexChars[(i >> 12) & 0xF];
				line[5] = HexChars[(i >> 8) & 0xF];
				line[6] = HexChars[(i >> 4) & 0xF];
				line[7] = HexChars[(i >> 0) & 0xF];

				int hexColumn = firstHexColumn;
				int charColumn = firstCharColumn;

				for (int j = 0; j < bytesPerLine; j++)
				{
					if (j > 0 && (j & 7) == 0) hexColumn++;
					if (i + j >= bytesLength)
					{
						line[hexColumn] = ' ';
						line[hexColumn + 1] = ' ';
						line[charColumn] = ' ';
					}
					else
					{
						byte b = bytes[i + j];
						line[hexColumn] = HexChars[(b >> 4) & 0xF];
						line[hexColumn + 1] = HexChars[b & 0xF];
						if(b < 32 || b >= 127)
							line[charColumn] = '·';
						else
							line[charColumn] = (char)b;
					}
					hexColumn += 3;
					charColumn++;
				}
				result.Append(line);
			}
			return result.ToString();
		}

		public class SimpleFolder
		{
			public List<string> fPath;
			public List<string> fName;
			public List<long> fSize;
			public int count { get; set; }

			public SimpleFolder(string baseDir)
			{
				fPath = new List<string>();
				fName = new List<string>();
				fSize = new List<long>();
				count = 0;
				ReadFolder(baseDir);
			}

			public void ReadFolder(string baseDir)
			{
				DirectoryInfo oDir = new DirectoryInfo(baseDir);
				FileInfo[] oFiles = oDir.GetFiles();

				foreach (FileInfo oFile in oFiles)
				{
					fSize.Add(oFile.Length);
					fPath.Add(oFile.FullName);
					fName.Add(oFile.Name);
					count++;
				}
			}
		}
	}
}
