﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace LaPucelleToolpack
{
	public partial class frmMain : Form
	{

		int Rows, Columns;
		Size CellSize;
		Point DrawCellPt;


		Dictionary dict = new Dictionary();
		Tools tools = new Tools();
		LZS lzs = new LZS();
		START_DAT dat;
		NISPACK nisPack;
		SCRIPT_TALK scriptTalk;
		TXP txp;
		FNT fnt;

		string sInput = "";
		string sOutput = "";

		public frmMain()
		{
			InitializeComponent();

			int value = (int)'Ａ';
			value = (int)'ａ';

			if (value == 0)
			{
			}


			this.Rows = (int)(picFont.Width / 14); //Anzahl der Zeilen des Rasters
			this.Columns = (int)(picFont.Height / 14);  //Anzahl der Spalten des Rasters

			//Aus Anzahl Spalten und Reihen resultierende Rasterzellgröße
			this.CellSize = new Size((int)(picFont.ClientSize.Width / this.Rows),
				(int)(this.picFont.ClientSize.Height / this.Columns));

		}

		private void frmMain_Load(object sender, EventArgs e)
		{

		}

		private void mnuUnpackStartJP_Click(object sender, EventArgs e)
		{
			if (tools.OpenFileDlg("*.LZS|*.LZS", "Open START_JP.LZS", ref sInput))
			{
				string outDir = Path.GetDirectoryName(sInput) + "\\" +
					Path.GetFileNameWithoutExtension(sInput) + "\\";

				byte[] data = null;
				data = lzs.decompressLZS(sInput);

				//debug
				//File.WriteAllBytes("test1.dat", data);

				dat = new START_DAT(tools);

				if (dat.ExtractFiles(data, outDir) == -1)
				{
					MessageBox.Show(dat.errorMsg, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
				}
				else
				{
					MessageBox.Show("Unpacked to '" + outDir + "'.", "Success",
						MessageBoxButtons.OK, MessageBoxIcon.Information);
				}
			}
		}

		private void mnuRepackStartJP_Click(object sender, EventArgs e)
		{
			this.Enabled = false;

			string outDir = tools.ChooseFolder("");
			if (outDir != "")
			{
				if (sInput != "")
					sInput = Path.GetFileName(sInput);
				else
					sInput = "new.lzs";

				if (tools.SaveFileDlg("*.LZS|*.LZS", "Save new START_JP.LZS", sInput, ref sOutput))
				{
					dat = new START_DAT(tools);
					//File.WriteAllBytes("test2.dat", dat.RepackFiles(outDir));
					File.WriteAllBytes(sOutput, lzs.compressLZS(dat.RepackFiles(outDir)));

					MessageBox.Show("Repacked into '" + sOutput + "'.", "Success",
						MessageBoxButtons.OK, MessageBoxIcon.Information);
				}
			}

			this.Enabled = true;
		}

		private void mnuUnpackScriptPack_Click(object sender, EventArgs e)
		{
			if (tools.OpenFileDlg("*.DAT|*.DAT", "Open SCRIPTPACK.DAT", ref sInput))
			{
				string outDir = Path.GetDirectoryName(sInput) + "\\" +
					Path.GetFileNameWithoutExtension(sInput) + "\\";

				nisPack = new NISPACK(tools);

				if (nisPack.ExtractFiles(sInput, outDir) == -1)
				{
					MessageBox.Show(nisPack.errorMsg, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
				}
				else
				{
					MessageBox.Show("Unpacked to '" + outDir + "'.", "Success", 
						MessageBoxButtons.OK, MessageBoxIcon.Information);
				}
			}
		}

		private void mnuRepackScriptPack_Click(object sender, EventArgs e)
		{
			string outDir = tools.ChooseFolder("");
			if (outDir != "")
			{
				if (sInput != "")
					sInput = Path.GetFileName(sInput);
				else
					sInput = "new.dat";

				if (tools.SaveFileDlg("*.DAT|*.DAT", "Save new SCRIPTPACK DAT", sInput, ref sOutput))
				{
					nisPack = new NISPACK(tools);
					nisPack.RepackFiles(outDir, sOutput);
					MessageBox.Show("Repacked into '" + sOutput + "'.", "Success",
						MessageBoxButtons.OK, MessageBoxIcon.Information);
				}
			}
		}

		private void mnuLoadScriptTalk_Click(object sender, EventArgs e)
		{
			if (tools.OpenFileDlg("talk*.dat|talk*.dat", "Open talk*.dat", ref sInput))
			{
				scriptTalk = new SCRIPT_TALK(tools);
				if (scriptTalk.ReadData(sInput) == -1)
				{
					MessageBox.Show(scriptTalk.errorMsg, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
				}
				lstEntrys.Items.Clear();
				for (int i = 0; i < scriptTalk.numEntries; i++)
				{
					if (scriptTalk.entries[i].flags[0] == 0)
						lstEntrys.Items.Add(string.Format("{0:D4} -> LBL {1}", i,
							scriptTalk.entries[i].flags[10]));
					else
						lstEntrys.Items.Add(string.Format("{0:D4} -> LBL {1}", i, 
							scriptTalk.entries[i].flags[0]));
				}
				this.Text = "La Pucelle - Tactics/Ragnarok - Toolpack [" + Path.GetFileName(sInput) + "]";
			}
		}

		private void mnuSaveScriptTalk_Click(object sender, EventArgs e)
		{
			if (scriptTalk != null && scriptTalk.numEntries > 0 && 
				tools.SaveFileDlg("talk*.dat|talk*.dat", "Save talk*.dat", Path.GetFileName(sInput), ref sOutput))
			{
				scriptTalk.WriteData(sOutput);
				MessageBox.Show("Repacked into '" + sOutput + "'.", "Success",
					MessageBoxButtons.OK, MessageBoxIcon.Information);
			}
		}

		private void mnuLoadTXP_Click(object sender, EventArgs e)
		{
			if (tools.OpenFileDlg("*.txp;*.tx2|*.txp;*.tx2", "Open TXP image", ref sInput))
			{
				txp = new TXP(tools);
				txp.bIsPS2 = optPS2.Checked == true;
				txp.bEnableSwizzle = optSwizzleEnabled.Checked == true;
				txp.bEnableAlpha = chkAlpha.Checked == true;
				txp.LoadTXP(sInput);

				this.picImage.Image = txp.image;
			}
		}

		private void mnuLoadPNG_Click(object sender, EventArgs e)
		{
			if (tools.OpenFileDlg("*.png|*.png", "Open PNG image", ref sInput))
			{
				this.picImage.Image = Image.FromFile(sInput);
			}
		}

		private void lstEntrys_SelectedIndexChanged(object sender, EventArgs e)
		{
			int x = lstEntrys.SelectedIndex;

			if (x > -1)
			{
				txtDebug.Text = "";

				for (int j = 0; j < 14; j++)
				{
					txtDebug.Text += string.Format("{0} - {1}; ", j, scriptTalk.entries[x].flags[j]);
					if(j % 2 == 1)
						txtDebug.Text += "\r\n";
				}
				txtDebug.Text += "\r\n";

				txtHexDump.Text = tools.HexDump(scriptTalk.entries[x].data);
				txtEditor.Text = "";
				txtEditor.Text = scriptTalk.ReadTextScript(scriptTalk.entries[x].data);
			}
		}

		private void btnCompile_Click(object sender, EventArgs e)
		{
			int x = lstEntrys.SelectedIndex;
			if (x > -1 && txtEditor.Text != "" && scriptTalk != null && scriptTalk.numEntries > 0)
			{
				scriptTalk.entries[x].data = scriptTalk.WriteTextScript(txtEditor.Text, this.chkPadding.Checked == true);
				txtHexDump.Text = tools.HexDump(scriptTalk.entries[x].data);
			}
		}

		private void btnReplace_Click(object sender, EventArgs e)
		{
			ReplaceMissing();
		}

		private void btnSaveImagePNG_Click(object sender, EventArgs e)
		{
			if (this.picImage.Image != null &&
				tools.SaveFileDlg("*.png|*.png", "Save PNG image", "image.png", ref sOutput))
			{
				this.picImage.Image.Save(sOutput, System.Drawing.Imaging.ImageFormat.Png);
				MessageBox.Show("Saved to '" + sOutput + "'.", "Success",
					MessageBoxButtons.OK, MessageBoxIcon.Information);
			}
		}

		private void btnSaveImageTXP_Click(object sender, EventArgs e)
		{
			if (this.picImage.Image != null &&
				tools.SaveFileDlg("*.txp;*.tx2|*.txp;*.tx2", "Save TXP image", "image.txp", ref sOutput))
			{
				txp = new TXP(tools);
				File.WriteAllBytes(sOutput, txp.SaveTXP(this.picImage.Image));
				MessageBox.Show("Saved to '" + sOutput + "'.", "Success",
					MessageBoxButtons.OK, MessageBoxIcon.Information);
			}
		}

		private void btnExportBMP_Click(object sender, EventArgs e)
		{
			if (tools.OpenFileDlg("*.txp;*.tx2|*.txp;*.tx2", "Open TXP image", ref sInput))
			{
				txp = new TXP(tools);
				txp.bIsPS2 = optPS2.Checked == true;
				txp.bEnableSwizzle = optSwizzleEnabled.Checked == true;
				txp.bEnableAlpha = chkAlpha.Checked == true;

				if (tools.SaveFileDlg("*.bmp|*.bmp", "Save BMP image", "image.bmp", ref sOutput))
				{
					txp.ExportBMP(sInput, sOutput);
				}
			}
		}

		private void btnImportBMP_Click(object sender, EventArgs e)
		{
			if (tools.OpenFileDlg("*.bmp|*.bmp", "Open BMP image", ref sInput))
			{
				txp = new TXP(tools);
				txp.bIsPS2 = optPS2.Checked == true;
				txp.bEnableSwizzle = optSwizzleEnabled.Checked == true;
				txp.bEnableAlpha = chkAlpha.Checked == true;

				if (tools.SaveFileDlg("*.txp;*.tx2|*.txp;*.tx2", "Save TXP image", "image.bmp", ref sOutput))
				{
					txp.ImportBMP(sInput, sOutput);
				}
			}
		}

		private void btnReplaceJASCII_Click(object sender, EventArgs e)
		{
			Dictionary dict = new Dictionary();

			string text = txtEditor.Text;

			text = dict.JASCII2ASCII(text);
			//text = text.Replace("．．．", "…");
			//text = text.Replace("...", "…");
			text = text.Replace("…", "...");
			text = text.Replace("'ve", "' ve");
			text = text.Replace("'re", "' re");
			text = text.Replace("'t", "' t");
			text = text.Replace("'bout", "' bout");
			text = text.Replace("'m", "' m");
			text = text.Replace("'ll", "' ll");
			text = text.Replace("'s", "' s");
			text = text.Replace("'d", "' d");

			txtEditor.Text = text;
		}

		private void ReplaceMissing()
		{
			string text = txtEditor.Text;
			string[] lines = text.Split(new string[] { "\r\n" }, StringSplitOptions.None);
			string[] lines2;
			string line;
			text = "";

			for (int i = 0; i < lines.Length; i++)
			{
				line = lines[i];
				if (line != "")
				{
					lines2 = line.Split(';');
                    if (lines2[0] == "\\Text")
                    {
                        lines2[1] = ReplaceMissing(lines2[1]);
                        lines[i] = lines2[0] + ";" + lines2[1];
                    }
                    text += lines[i] + "\r\n";
				}
				
			}
			txtEditor.Text = text;
		}

		private string ReplaceMissing(string text)
		{
			string JASCII = "７９ｑｘｙＱＶ！？。。、、””（）";
			string ASCII = "79qxyQV!?.．,，'\"()";

			for (int i = 0; i < ASCII.Length; i++)
			{
				text = text.Replace(ASCII[i], JASCII[i]);
			}
			return text;
		}

		private void btnLoadFontImage_Click(object sender, EventArgs e)
		{
			if (tools.OpenFileDlg("*.png|*.png", "Open PNG image", ref sInput))
			{
				this.picFont.Image = Image.FromFile(sInput);
			}
		}

		private void btnLoadFontFile_Click(object sender, EventArgs e)
		{
			if (tools.OpenFileDlg("fontB.fnt|fontB.fnt", "Open fontB.fnt", ref sInput))
			{
				fnt = new FNT(tools);
				fnt.LoadFNT(sInput);

				if (fnt.LoadFNT(sInput) != -1)
				{
					lstLetters.Items.Clear();

					for (int i = 0; i < fnt.numEntries; i++)
					{
						lstLetters.Items.Add(string.Format("{0:D5}\t{1}\t{2}", i, fnt.letters[i], fnt.entries[i]));
					}
				}
			}
		}

		private void btnSaveFontFile_Click(object sender, EventArgs e)
		{
			if (fnt != null && tools.SaveFileDlg("fontB.fnt|fontB.fnt", "Save font file", "fontB.fnt", ref sOutput))
			{
				if (fnt.SaveFNT(sOutput) != -1)
				{
					MessageBox.Show("Saved to '" + sOutput + "'.", "Success",
						MessageBoxButtons.OK, MessageBoxIcon.Information);
				}
			}
		}

		private void picFont_MouseMove(object sender, MouseEventArgs e)
		{
			Point TmpPoint = DrawCellPt;

			// columns
			for (int i = 0; i < picFont.ClientSize.Height; i += CellSize.Height)
			{
				if (e.Y >= i && e.Y <= i + CellSize.Height)
				{
					this.DrawCellPt.Y = i;
				}
			}

			// rows
			for (int i = 0; i < picFont.ClientSize.Width; i += CellSize.Width)
			{
				if (e.X >= i && e.X <= i + CellSize.Width)
				{
					this.DrawCellPt.X = i;
				}
			}

	        if(TmpPoint.X != DrawCellPt.X || TmpPoint.Y != DrawCellPt.Y)
				picFont.Invalidate();
		}

		private void picFont_Paint(object sender, PaintEventArgs e)
		{
			Graphics g = e.Graphics;
			g.DrawRectangle(new Pen(Color.White, 1), 
				new Rectangle(DrawCellPt.X, DrawCellPt.Y, CellSize.Width, CellSize.Height));
		}

		private void picFont_MouseClick(object sender, MouseEventArgs e)
		{
			int i = lstLetters.SelectedIndex;

			if (i > -1 && fnt != null)
			{
				int x = (int)(e.X / 14);
				int y = (int)(e.Y / 14);

				fnt.entries[i] = (ushort)((y * 36) + x);

				lstLetters.Items[i] = string.Format("{0:D5}\t{1}\t{2}", i, fnt.letters[i], fnt.entries[i]);
			}
		}

        private void replacePunc_Click(object sender, EventArgs e)
        {
            txtEditor.Text = txtEditor.Text.Replace("，", "、");
            txtEditor.Text = txtEditor.Text.Replace("．", "。");
            txtEditor.Text = txtEditor.Text.Replace(" ", "　");
        }
        private void replaceASCII_Click(object sender, EventArgs e)
        {
            Dictionary dict = new Dictionary();
            string text = txtEditor.SelectedText;
            string text1 = dict.ASCII2JASCII(text);
            txtEditor.SelectedText = txtEditor.SelectedText.Replace(text, text1);
        }

        private void tabPicture_Click(object sender, EventArgs e)
        {

        }

        private void button1_Click(object sender, EventArgs e)
        {
            string text = txtEditor.Text;
            string[] lines = text.Split(new string[] { "\r\n" }, StringSplitOptions.None);
            string[] scriptLine;
            string charLength;
            CharCount.Clear();
            foreach (string s in lines)
            {
                if (s.Contains(";"))
                {
                    scriptLine = s.Split(';');
                    charLength = scriptLine[1].Length.ToString() + "\r\n";
                    CharCount.AppendText(charLength);
                }
            }
        }

        private void batchTXPtoPNG_MouseClick(object sender, MouseEventArgs e)
        {
            string outDir = tools.ChooseFolder("");
            if (outDir != "")
            {
                string[] filePaths = Directory.GetFiles(outDir, "*.txp");

                foreach (string filePathName in filePaths)
                {
                    txp = new TXP(tools);
                    txp.bIsPS2 = optPS2.Checked == true;
                    txp.bEnableSwizzle = optSwizzleEnabled.Checked == true;
                    txp.bEnableAlpha = chkAlpha.Checked == true;
                    txp.LoadTXP(filePathName);
                    this.picImage.Image = txp.image;
                    string filePathPNG = filePathName.Replace(".txp", ".png");
                    this.picImage.Image.Save(filePathPNG, System.Drawing.Imaging.ImageFormat.Png);
                }
                MessageBox.Show("Batch TXP to PNG Complete", "Success", MessageBoxButtons.OK, MessageBoxIcon.Information);
                this.picImage.Image = null;
            }
            
                
        }

	}
}
